/*******************************************************************************
PolSARpro v2.0 is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 (1991) of the License, or any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. 

See the GNU General Public License (Version 2, 1991) for more details.

********************************************************************************

	File	 : alos_header.h
	Project  : ESA_POLSARPRO
	Authors  : Eric POTTIER
	Version  : 1.0
	Creation : 08/2006
	Update	:

*-------------------------------------------------------------------------------
	INSTITUT D'ELECTRONIQUE et de TELECOMMUNICATIONS de RENNES (I.E.T.R)
	UMR CNRS 6164																						  
	Groupe Image et Teledetection
	Equipe SAPHIR (SAr Polarimetrie Holographie Interferometrie Radargrammetrie)
	UNIVERSITE DE RENNES I
	Ple Micro-Ondes Radar
	Bt. 11D - Campus de Beaulieu
	263 Avenue Gnral Leclerc
	35042 RENNES Cedex
	Tel :(+33) 2 23 23 57 63
	Fax :(+33) 2 23 23 69 63
	e-mail : eric.pottier@univ-rennes1.fr
*-------------------------------------------------------------------------------
	Description :  ALOS CEOS Header File

	This file includes definitions and constants used by alos_header.c

	This software is extracted and adapted from the SIR-C GDPS OPS ceos_rd.c software
	Copyright (c) 1993, California Institute of Technology. U.S.
	Government Sponsorship under NASA Contract NAS7-918 is acknowledged.
	Author:	P. Barrett, Initiated: 11-FEB-93

	The updates have been made according to the document:
	"ALOS/PALSAR Level 1 product - Format Description, vol2, level 1.1/1.5"
	Revision G, October 2005, JAXA, Earth Observation Researcg and Application Center

******************************************************************************/

#ifndef ALOS_CEOS_RD_H
#define ALOS_CEOS_RD_H

/* Miscellaneous */
#define SPACE '\040'  /* ascii space character */

/********************************************************************
*																	*
*								Titles								*
*																	*
********************************************************************/


#define STARS1 "***************************************************"
#define STARS2 "*                                                 *"
#define DATAS1 "*** Data Set"
#define DATAS2 "***"
#define TABLE1 "*** Table Entry"
#define TABLE2 "***"

#define SARL_TITLE		"*  SAR Leader File:"
#define IMG_TITLE		"*  Imagery Options File:"
#define SART_TITLE		"*  SAR Trailer File:"

#define DESCR_REC		"*  File Descriptor Record"
#define PTR_REC_SARL	"*  File Pointer Record: SAR Leader"
#define PTR_REC_IMG		"*  File Pointer Record: Imagery Options"
#define PTR_REC_SART	"*  File Pointer Record: SAR Trailer"
#define TEXT			"*  Text Record"

#define DSS_REC			"*  Data Set Summary Record"
#define MAP_REC			"*  Map Projection Data Record"
#define PLATF_REC		"*  Platform Position Record"
#define ATT_REC			"*  Attitude Data Record"
#define RADIO_REC		"*  Radiometric Data Record"
#define QUAL_REC		"*  Data Quality Summary Record"
#define FACIL_REC		"*  Facility Data Record "
#define PREFIX_REC		"*  First Prefix Data Record"

/* constants */

/*#define MAX_MISSING_PAIRS	 5*/  /* Max pairs of missing rg lines/chan */
#define NAME_LEN			128	 /* short string length */
#define LONG_NAME_LEN		128	 /* long string length */
#define MAX_POLS			4	 /* max number of pols: HH, HV, VV, VH */

/********************************************************************
*																	*
*							Text Fields								*
*																	*
********************************************************************/

/* Common Header */

#define HDR_SEQ		"Record Sequence Number:"
#define HDR_1SUB	"First Record Subtype Code:"
#define HDR_TYPE	"Record Type Code:"
#define HDR_2SUB	"Second Record Subtype Code:"
#define HDR_3SUB	"Third Record Subtype Code:"
#define HDR_LEN		"Record Length:"

/* SAR Leader: File Descriptor Headers */

#define ASCII_FL	"Data Format (A = ASCII):"
#define DOC_FORM	"Format Control Document:"
#define FORM_REV	"Format Control Document Version:"
#define REC_REV		"Record Format Revision Level:"
#define FILE_NO		"File Number:"
#define FILE_NAME	"File Name:"
#define FSEQ		"Record Sequence and Location Type Flag:"
#define LOC_NO		"Sequence Number Location:"
#define SEQ_FLD_L	"Sequence Number Field Length:"
#define LOC_TYPE	"Record Code and Location Type Flag:"
#define RCODE_LOC	"Record Code Location:"
#define RCODE_FLD_L	"Record Code Field Length:"
#define FLGT		"Record Length and Location Type Flag:"
#define REC_LEN_LOC	"Record Length Location:"
#define REC_LEN_LEN	"Record Length Field Length:"
#define RECORD_LEN	"Record Length (bytes):"
#define DS_FILE		"Number of Data Set Summary Records:"
#define MP_FILE		"Number of Map Projection Records:"
#define PLATF_FILE	"Number of Platform Position Records:"
#define ATT_FILE	"Number of Attitude Records:"
#define RADIO_FILE	"Number of Radiometric Data Records:"
#define RADIOC_FILE	"Number of Radiometric Compensation Records:"
#define QUAL_FILE	"Number of Data Quality Summary Records:"
#define HIST_FILE	"Number of Data Histogram Records:"
#define SPEC_FILE	"Number of Range Spectra Records:"
#define ELEV_FILE	"Number of Digital Elevation Model Records:"
#define UPD_FILE	"Number of Radar Parameter Update Records:"
#define ANNOT_FILE	"Number of Annotation Data Records:"
#define PROC_FILE	"Number of Detailed Processing Records:"
#define CAL_FILE	"Number of Calibration Data Records:"
#define GROUND_FILE	"Number of Ground Control Points Records:"
#define FAC_FILE1	"Number of Facility Data Record #01:"
#define RECORD_LEN1	"Record Length (bytes) #01:"
#define FAC_FILE2	"Number of Facility Data Record #02:"
#define RECORD_LEN2	"Record Length (bytes) #02:"
#define FAC_FILE3	"Number of Facility Data Record #03:"
#define RECORD_LEN3	"Record Length (bytes) #03:"
#define FAC_FILE4	"Number of Facility Data Record #04:"
#define RECORD_LEN4	"Record Length (bytes) #04:"
#define FAC_FILE5	"Number of Facility Data Record #05:"
#define RECORD_LEN5	"Record Length (bytes) #05:"
#define FAC_FILE6	"Number of Facility Data Record #06:"
#define RECORD_LEN6	"Record Length (bytes) #06:"
#define FAC_FILE7	"Number of Facility Data Record #07:"
#define RECORD_LEN7	"Record Length (bytes) #07:"
#define FAC_FILE8	"Number of Facility Data Record #08:"
#define RECORD_LEN8	"Record Length (bytes) #08:"
#define FAC_FILE9	"Number of Facility Data Record #09:"
#define RECORD_LEN9	"Record Length (bytes) #09:"
#define FAC_FILE10	"Number of Facility Data Record #10:"
#define RECORD_LEN10 "Record Length (bytes) #10:"
#define FAC_FILE11	"Number of Facility Data Record #11:"
#define RECORD_LEN11 "Record Length (bytes) #11:"

/* SAR Leader: Data Set Summary Headers */

#define SEQ_NO		"Sequence Number:"
#define SAR_CHAN	"SAR Channel Indicator:"
#define SITE_ID		"Site ID:"
#define SITE_NAME	"Site Name:"
#define CTR_GMT		"GMT, Image Center (YYYY/MM/DD hh:mm:ss.ttt):"
#define CTR_MET		"MET, Image Center (DD hh:mm:ss.ttt):"
#define LAT_CTR		"Geodetic Latitude at Image Center (deg):"
#define LONG_CTR	"Geodetic Longitude at Image Center (deg):"
#define TRK_ANG_CTR	"Track Angle at Image Center (deg):"
#define ELLIPSE_NAME "Ellipsoid Designator:"
#define MAJOR_AXIS	"Ellipsoid Semimajor Axis (km):"
#define MINOR_AXIS	"Ellipsoid Semiminor Axis (km):"
#define EARTH_MASS	"Earth's Mass:"
#define GRAVIT_CST	"Gravitational Constant:"
#define ELLIPSOIDJ2	"Ellipsoid J2 Parameter:"
#define ELLIPSOIDJ3	"Ellipsoid J3 Parameter:"
#define ELLIPSOIDJ4	"Ellipsoid J4 Parameter:"
#define TERR_HT		"Average Terrain Height above Ellipsoid (m):"
#define CTR_LINE	"Image Center Line Number:"
#define CTR_PIX		"Image Center Pixel Number:"
#define IMG_LEN		"Image Length (km):"
#define IMG_WID		"Image Width (km):"
#define N_CHAN		"Number of SAR Polarization Channels:"
#define PLAT_ID		"Sensor Platform Mission ID:"
#define SENSE_ID	"Sensor ID:"
#define BAND		"SAR Band:"
#define RESOL		"Resolution (HI = 20 MHz, LO = 10 MHz):"
#define ACQ			"Data Acquisition Mode:"
#define TX_POL		"Transmit Polarization:"
#define RX_POL		"Receive Polarization:"
#define DT_ID		"Datatake ID:"
#define CRAFT_LAT	"Platform Geodetic Latitude at Nadir (deg):"
#define CRAFT_LONG	"Platform Geodetic Longitude at Nadir (deg):"
#define PLAT_HD		"Platform Heading at Nadir (deg):"
#define CLOCK_ANG	"Clock Angle (Left: -90, Right: +90) (deg):"
#define INC_ANGLE	"Incidence Angle at Image Center (deg):"
#define WAVE_LEN	"Radar Wavelength (m):"
#define MOTION		"Motion Compensation (00 = No Compensation):"
#define CD_SPEC		"Range Pulse Code Specifier:"
#define CHIRP_A1	"Range Chirp Amplitude Coefficient #1:"
#define CHIRP_A2	"Range Chirp Amplitude Coefficient #2:"
#define CHIRP_A3	"Range Chirp Amplitude Coefficient #3:"
#define CHIRP_A4	"Range Chirp Amplitude Coefficient #4:"
#define CHIRP_A5	"Range Chirp Amplitude Coefficient #5:"
#define CHIRP_P1	"Range Chirp Phase Coefficient #1:"
#define CHIRP_P2	"Range Chirp Phase Coefficient #2:"
#define CHIRP_P3	"Range Chirp Phase Coefficient #3:"
#define CHIRP_P4	"Range Chirp Phase Coefficient #4:"
#define CHIRP_P5	"Range Chirp Phase Coefficient #5:"
#define DOWN_LINK	"Down Link Data Chirp Extraction Index:"
#define RG_SAMPL	"Range Complex Sampling Rate (MHz):"
#define ECHO_DLAY	"One-Way Echo Delay Time (microsec):"
#define PULSE_LEN	"Range Pulse Duration (microsec):"
#define BAND_CONV	"Base Band Conversion ?:"
#define COMPR_FLAG	"Range Compressed ?:"
#define LIKE_GAIN	"Receiver Gain for Like Polarization (dB):"
#define CROSS_GAIN	"Receiver Gain for Cross Polarization (dB):"
#define Q_BITS		"Quantization in Bits per Channel:"
#define QUANT_DES	"Quantization Descriptor:"
#define BIAS_I		"DC Biais I Component:"
#define BIAS_Q		"DC Biais Q Component:"
#define GAIN_IM		"Gain Imbalance I & Q Components:"
#define ELECT_BORE	"Antenna Electronic Boresight (deg):"
#define MECH_BORE	"Antenna Mechanical Boresight (deg):"
#define ECHO_TRK	"Echo Tracking:"
#define PRF			"Nominal PRF (mHz):"
#define RG_BEAMW	"Antenna Elevation 3 dB Beamwidth (deg):"
#define AZ_BEAMW	"Antenna Azimuth 3 dB  Beamwidth (deg):"
#define SAT_TIME	"Satellite Encoded Binary Time Code:"
#define SAT_CLOCK	"Satellite Clock Time:"
#define SAT_INCR	"Satellite Clock Increment:"
#define HW_VERS		"Hardware Version:"
#define SW_VERS		"Software Version:"
#define PROC_FAC	"Processing Facility:"
#define PROC_CODE	"Process Code:"
#define PROD_CODE	"Product Level Code:"
#define PROD_TYPE	"Product Type:"
#define PROC_ALG	"Processing Algorithm:"
#define N_LOOKS_AZ	"Total Number of Looks in Azimuth:"
#define N_LOOKS_RG	"Total Number of Looks in Range:"
#define BNDWTH_AZ	"Bandwidth per Look in Azimuth (Hz):"
#define BNDWTH_RG	"Bandwidth per Look in Range (Hz):"
#define AZ_BNDWTH	"Total Processor Azimuth Bandwidth (Hz):"
#define RG_BNDWTH	"Total Processor Range Bandwidth (kHz):"
#define AZ_WEIGHT	"Azimuth Weighting Function:"
#define RG_WEIGHT	"Range Weighting Function:"
#define HDDC		"HDDC ID:"
#define RG_RES		"Nominal Range Resolution (m):"
#define AZ_RES		"Nominal Azimuth Resolution (m):"
#define GAIN		"Processor Gain for Noise Data:"
#define BIAS		"Linear Radiometric Conversion Factor:"
#define AL_DOPPLER1	"Along Track Doppler Constant Term:"
#define AL_DOPPLER2	"Along Track Doppler Linear Term:"
#define AL_DOPPLER3	"Along Track Doppler Quadratic Term:"
#define AC_DOPPLER1	"Across Track Doppler Constant Term:"
#define AC_DOPPLER2	"Across Track Doppler Linear Term:"
#define AC_DOPPLER3	"Across Track Doppler Quadratic Term:"
#define RG_TIME		"Time Direction Indicator (Range):"
#define AZ_TIME		"Time Direction Indicator (Azimuth):"
#define AL_DOPPLER4	"Along Track Doppler Rate Constant Term:"
#define AL_DOPPLER5	"Along Track Doppler Rate Linear Term:"
#define AL_DOPPLER6	"Along Track Doppler Rate Quadratic Term:"
#define AC_DOPPLER4	"Across Track Doppler Rate Constant Term:"
#define AC_DOPPLER5	"Across Track Doppler Rate Linear Term:"
#define AC_DOPPLER6	"Across Track Doppler Rate Quadratic Term:"
#define LINE_CON	"Line Content:"
#define CLUTTER		"Clutter Lock Applied ?:"
#define AUTOFOCUS	"Autofocussing Applied ?:"
#define LINE_SP		"Line Spacing (m):"
#define PIXEL_SP	"Pixel Spacing (m):"
#define RG_COMPR	"Processor Range Compression:"
#define DOPPLER_F1	"Doppler Center Frequency Constant term:"
#define DOPPLER_F2	"Doppler Center Frequency Linear term:"
#define CALIB_DATA	"Calibration Data Indicator:"
#define CALIB_LINE1	"Start Line Number of Calib at Upper Image:"
#define CALIB_LINE2	"Stop Line Number of Calib at Upper Image:"
#define CALIB_LINE3	"Start Line Number of Calib at Bottom Image:"
#define CALIB_LINE4	"Stop Line Number of Calib at Bottom Image:"
#define PRF_INDIC	"PRF Switching Indicator:"
#define PRF_LINE	"Line Indicator of PRF Switching:"
#define BEAM_CENTER	"Direction of the Beam Center in a Scene Center:"
#define YEW_STEERING "Yew Steering Mode Flag:"
#define PARA_NUMBER	"Parameter Table Number of Automatic Setting:"
#define OFFNADIR_ANG "Nominal Off-Nadir Angle:"
#define BEAM_NUMBER	"Antenna Beam Number:"
#define INC_ANG_A0	"Incidence Angle Constant Term:"
#define INC_ANG_A1	"Incidence Angle Linear Term:"
#define INC_ANG_A2	"Incidence Angle Quadratic Term:"
#define INC_ANG_A3	"Incidence Angle Cubic Term:"
#define INC_ANG_A4	"Incidence Angle Fourth Term:"
#define INC_ANG_A5	"Incidence Angle Fifth Term:"
#define N_ANNOT		"Number of Annotation Points:"
#define ANNOT_LINE	"Line Number of 1st Annotation Start:"
#define ANNOT_PIXEL	"Pixel Number of 1st Annotation Start:"
#define ANNOT_TEXT	"1st Annotation Text:"
#define ANNOT_TEXT2	"2nd to 64th Annotation Text:"

/* SAR Leader: Map Projection Headers */

#define MP_DESCR	"Map Projection Descriptor:"
#define N_PIXELS	"Number of Pixels per Image Line:"
#define N_LINES		"Number of Image Lines:"
#define ORIENT		"Orientation Output Scene (deg):"
#define PL_INCL		"Platform Inclinaison (deg):"
#define AS_NODE		"Ascending Node (deg):"
#define PL_DIST		"Platform Distance at Image Center (m):"
#define ALTITUDE	"Geodetic Altitude of Platform (m):"
#define SPEED		"Spacecraft Speed at Nadir (m/sec):"
#define MAJOR_AXIS_M "Ellipsoid Semimajor Axis (m):"
#define MINOR_AXIS_M "Ellipsoid Semiminor Axis (m):"
#define DATUM_SHIFT1 "Datum Shift Para Referenced to Greenwitch:"
#define DATUM_SHIFT2 "Datum Shift Para Perpendicular to Greenwitch:"
#define DATUM_SHIFT3 "Datum Shift Para Direction of the Rotation:"
#define DATUM_SHIFT4 "Datum Shift Para 1st Rotation:"
#define DATUM_SHIFT5 "Datum Shift Para 2nd Rotation:"
#define DATUM_SHIFT6 "Datum Shift Para 3rd Rotation:"
#define SCALE1		"Scale Factor of Reference Ellipsoid:"
#define MAP_PROJ	"Map Projection:"
#define UTM_DESC	"UTM Descriptor:"
#define UTM_ZONE	"UTM Zone:"
#define MAP_FE1		"Map Origin - False Easting (m):"
#define MAP_FN1		"Map Origin - False Northing (m):"
#define CTR_PRJ_LO1	"UTM Center of Projection Longitude (deg):"
#define CTR_PRJ_LA1	"UTM Center of Projection Latitude (deg):"
#define STAND_PARA1	"1st Standard Parallel:"
#define STAND_PARA2	"2nd Standard Parallel:"
#define SCALE2		"UTM Scale Factor:"
#define UPS_DESC	"UPS Descriptor:"
#define CTR_PRJ_LO2	"UPS Center of Projection Longitude (deg):"
#define CTR_PRJ_LA2	"UPS Center of Projection Latitude (deg):"
#define SCALE3		"UPS Scale Factor:"
#define PROJ_DESC	"Projection Descriptor:"
#define MAP_FE2		"NSP - Map Origin - False Easting (m):"
#define MAP_FN2		"NSP - Map Origin - False Northing (m):"
#define CTR_PRJ_LO3	"NSP Center of Projection Longitude (deg):"
#define CTR_PRJ_LA3	"NSP Center of Projection Latitude (deg):"
#define STAND_PARA3	"NSP - 1st Standard Parallel:"
#define STAND_PARA4	"NSP - 2nd Standard Parallel:"
#define NE_NO		"Top Left Corner Northing (km):"
#define NE_EA		"Top Left Corner Easting (km):"
#define FE_NO		"Top Right Corner Northing (km):"
#define FE_EA		"Top Right Corner Easting (km):"
#define FL_NO		"Bottom Right Corner Northing (km):"
#define FL_EA		"Bottom Right Corner Easting (km):"
#define NL_NO		"Bottom Left Corner Northing (km):"
#define NL_EA		"Bottom Left Corner Easting (km):"
#define NE_LAT		"Near Range Early Time Latitude (deg):"
#define NE_LONG		"Near Range Early Time Longitude (deg):"
#define FE_LAT		"Far Range Early Time Latitude (deg):"
#define FE_LONG		"Far Range Early Time Longitude (deg):"
#define FL_LAT		"Far Range Late Time Latitude (deg):"
#define FL_LONG		"Far Range Late Time Longitude (deg):"
#define NL_LAT		"Near Range Late Time Latitude (deg):"
#define NL_LONG		"Near Range Late Time Longitude (deg):"
#define COEFF_A11	"Coefficients A11:"
#define COEFF_A12	"Coefficients A12:"
#define COEFF_A13	"Coefficients A13:"
#define COEFF_A14	"Coefficients A14:"
#define COEFF_A21	"Coefficients A21:"
#define COEFF_A22	"Coefficients A22:"
#define COEFF_A23	"Coefficients A23:"
#define COEFF_A24	"Coefficients A24:"
#define COEFF_B11	"Coefficients B11:"
#define COEFF_B12	"Coefficients B12:"
#define COEFF_B13	"Coefficients B13:"
#define COEFF_B14	"Coefficients B14:"
#define COEFF_B21	"Coefficients B21:"
#define COEFF_B22	"Coefficients B22:"
#define COEFF_B23	"Coefficients B23:"
#define COEFF_B24	"Coefficients B24:"

/* SAR Leader: Platform Position Headers */

#define ORBIT_DESIGN "Orbital Elements Designator:"
#define ORBIT_ELT1	"1st Orbital Element:"
#define ORBIT_ELT2	"2nd Orbital Element:"
#define ORBIT_ELT3	"3rd Orbital Element:"
#define ORBIT_ELT4	"4th Orbital Element:"
#define ORBIT_ELT5	"5th Orbital Element:"
#define ORBIT_ELT6	"6th Orbital Element:"
#define PLAT_YEAR	"Year of First Point (YYYY):"
#define PLAT_MONTH	"Month of First Point (MM):"
#define PLAT_DAY	"Day of First Point (DD):"
#define PLAT_DIY	"Day of Year of First Point:"
#define PLAT_SID	"Seconds in Day of First Point:"
#define PLAT_INT	"Time Between Points (s):"
#define PLAT_COORD	"Reference Coordinate System:"
#define PLAT_HOUR	"Mean Hour Angle (deg):"
#define AL_TRACK_ERR1 "Along Track Position Error (m):"
#define AC_TRACK_ERR1 "Across Track Position Error (m):"
#define RD_TRACK_ERR1 "Radial Track Position Error (m):"
#define AL_TRACK_ERR2 "Along Track Velocity Error (m/s):"
#define AC_TRACK_ERR2 "Across Track Velocity Error (m/s):"
#define RD_TRACK_ERR2 "Radial Track Velocity Error (m/s):"
#define PLAT_POSV	"Position Vector (km):"
#define PLAT_VEL	"Velocity Vector (km/sec):"
#define PLAT_XYZ	"	x		y		 z"
#define OCC_FLAG	"Occurence Flag:"

/* SAR Leader: Attitude Data Headers */

#define ATT_DAYS	"GMT Day of Year:"
#define ATT_SECS	"GMT Millisecond of Day:"
#define PITCH_FLAG	"Pitch Data Quality Flag:"
#define ROLL_FLAG	"Roll Data Quality Flag:"
#define YAW_FLAG	"Yaw Data Quality Flag:"
#define PITCH_VAL	"Pitch Value (deg):"
#define ROLL_VAL	"Roll Value (deg):"
#define YAW_VAL		"Yaw Value (deg):"
#define PITCH_QUAL	"Pitch Rate Data Quality Flag:"
#define ROLL_QUAL	"Roll Rate Data Quality Flag:"
#define YAW_QUAL	"Yaw Rate Data Quality Flag:"
#define PITCH_RATE	"Pitch Rate Value (deg/s):"
#define ROLL_RATE	"Roll Rate Value (deg/s):"
#define YAW_RATE	"Yaw Rate Value (deg/s):"

/* SAR Leader: Radiometric Data Headers */

#define N_SETS		"Number of Data Sets:"
#define CAL_FAC		"Calibration Factor:"
#define DT11_R		"Real Part of Transmit Dist. Matrix DT(1,1):"
#define DT11_I		"Imag Part of Transmit Dist. Matrix DT(1,1):"
#define DT12_R		"Real Part of Transmit Dist. Matrix DT(1,2):"
#define DT12_I		"Imag Part of Transmit Dist. Matrix DT(1,2):"
#define DT21_R		"Real Part of Transmit Dist. Matrix DT(2,1):"
#define DT21_I		"Imag Part of Transmit Dist. Matrix DT(2,1):"
#define DT22_R		"Real Part of Transmit Dist. Matrix DT(2,2):"
#define DT22_I		"Imag Part of Transmit Dist. Matrix DT(2,2):"
#define DR11_R		"Real Part of Receive  Dist. Matrix DR(1,1):"
#define DR11_I		"Imag Part of Receive  Dist. Matrix DR(1,1):"
#define DR12_R		"Real Part of Receive  Dist. Matrix DR(1,2):"
#define DR12_I		"Imag Part of Receive  Dist. Matrix DR(1,2):"
#define DR21_R		"Real Part of Receive  Dist. Matrix DR(2,1):"
#define DR21_I		"Imag Part of Receive  Dist. Matrix DR(2,1):"
#define DR22_R		"Real Part of Receive  Dist. Matrix DR(2,2):"
#define DR22_I		"Imag Part of Receive  Dist. Matrix DR(2,2):"

/* SAR Leader: Data Quality Summary Headers */

#define BAND_POL	"SAR Frequency / Polarisation  :"
#define CAL_DATE	"Nearest Calibration Update (YYMMDD):"
#define ISLR		"Two-Dimensional ISLR (dB):"
#define PSLR		"Two-Dimensional PSLR (dB):"
#define AZ_AMB		"Nominal Azimuth Ambiguity (dB):"
#define RG_AMB		"Nominal Range Ambiguity (dB):"
#define SNR_EST		"SNR Estimate (from range spectra):"
#define BER			"Bit Error Rate Estimate:"
#define SLANT_RES	"Nominal Slant Range Resolution (m):"
#define RADIO_RES	"Nominal Radiometric Resolution (dB):"
#define DYNAMIC		"Instantaneous Dynamic Range (dB):"
#define ABS_RAD_UNC	"Abs. Radiometric Calibration Unc (dB):"
#define PHA_RAD_UNC	"Phase Radiometric Calibration Unc (deg):"
#define REL_RAD_UNC	"Relative Radio. Calib. Unc Channel #"
#define REL_PHA_UNC	"Relative Phase  Calib. Unc Channel #"
#define ALOC_E		"Abs. Location Error Along Track (m):"
#define XLOC_E		"Abs. Location Error Cross Track (m):"
#define ASCALE_E	"Geometric Scale Error Along Track:"
#define XSCALE_E	"Geometric Scale Error Cross Track:"
#define A_TRACK_MIS	"Along  Track Rel. Mis. Err. Channel #"
#define X_TRACK_MIS	"Across Track Rel. Mis. Err. Channel #"
#define SKEW_E		"Nominal Geometric Skew Error:"
#define ORIENT_E	"Scene Orientation Error (deg):"

/* SAR Leader: Facility Data Record 11 Headers */

#define COEFF_MAP	"Map Projection Coefficient "
#define CALIB_INDIC	"Calibration Indicator:"
#define START_UP	"Calib. Start Line Upper  Image:"
#define STOP_UP		"Calib. Stop  Line Upper  Image:"
#define START_BO	"Calib. Start Line Bottom Image:"
#define STOP_BO		"Calib. Stop  Line Bottom Image:"
#define PRF_IND		"PRF Switching Indicator:"
#define PRF_LIN		"PRF Switching Line Indicator:"
#define SAR_PROC	"SAR Processing Start Line Number:"
#define LOSS_LINE	"Number of Loss Lines:"
#define LOSS_LINE_RG "Number of Loss Range Lines:"

/* Imagery Options File: File Descriptor Record Headers */

#define N_SAR_RECS	"SAR Data Record Count:"
#define SAR_REC_LEN	"SAR Record Length (Bytes):"
#define BITS_SPAMP	"Bits Per Sample/Pixel (RSD & MLD Only):"
#define PIX_GP		"Pixels Per Data Group:"
#define BYTES_GP	"Bytes Per Data Group:"
#define N_SAR_CHAN	"Number of SAR Polarization Channels:"
#define N_LINES_IMG	"Number of Image Lines this Channel:"
#define LEFT_BD_PIX	"Left Border Pixels per Line:"
#define N_PIX_LINE	"Samples/Pixels/Data Groups per Line:"
#define RIGHT_BD_PIX "Right Border Pixels per Line:"
#define TOP_LINES	"Top Border Scan Lines:"
#define BOTTOM_LINES "Bottom Border Scan Lines:"
#define INTERLEAVE	"Interleaving Indicator:"
#define RECS_LINE	"Physical Records Per Line:"
#define RECS_CHAN	"Physical Records Per Multi-Channel Line:"
#define LINE_PREFIX	"Length of Prefix Data per Line (Bytes):"
#define BYTES_LINE	"Bytes of Sample/Image Data per Line:"
#define LINE_SUFFIX	"Length of Suffix Data per Line (Bytes):"
#define PREFIX_FLAG	"Prefix / Suffix Repeat Flag:"
#define LINE_LOC	"Sample Data Line Number Locator:"
#define SAR_LOC		"SAR Channel Number Locator:"
#define TIME_LOC	"Time of SAR Data Line Locator:"
#define LEFT_LOC	"Left-fill Count Locator:"
#define RIGHT_LOC	"Right-fill Count Locator:"
#define PAD_PIX		"Pad Pixels Present Indicator:"
#define DATA_LOC	"SAR Data Line Quality Code Locator:"
#define CALIB_LOC	"Calibration Information Field Vector:"
#define GAIN_LOC	"Gain Values Field Vector:"
#define BIAS_LOC	"Bias Values Field Locator:"
#define SAR_FORMAT	"SAR Data Format Indicator:"
#define SAR_CODE	"SAR Data Format Code:" 
#define LT_PIX_FILL	"Left Fill Bits within Pixel:"
#define RT_PIX_FILL	"Right Fill Bits within Pixel:"
#define MAX_RANGE	"Maximum Data Range of Pixel:"

/* Imagery Options File: Signal Data Record Headers */

#define SAR_LINE	"SAR Image Data Line Number:"
#define SAR_INDEX	"SAR Image Data Record Index:"
#define LEFT_PIX	"Actual Count of Left-Fill Pixels:"
#define DATA_PIX	"Actual Count of Data Pixels:"
#define RIGHT_PIX	"Actual Count of Right-Fill Pixels:"
#define SENSOR_FLAG	"Sensor Parameters Update Flag:"
#define SENSOR_YEAR	"Sensor Acquisition Year:"
#define SENSOR_DAY	"Sensor Acquisition Day of the Year:"
#define SENSOR_MS	"Sensor Acquisition Milliseconds of the Day:"
#define SAR_CHANNEL	"SAR Channel Indicator:"
#define SAR_CODE_I	"SAR Channel Code:"
#define T_POL		"Transmit Polarization:"
#define R_POL		"Receive Polarization:"
#define PRF_I		"PRF (mHz) :"
#define SCAN_ID		"Scan-Id for Scan Mode:"
#define RANGE_FLAG	"Onboard Range Compressed Flag:"
#define CHIRP_DES	"Chirp Type Designator:"
#define CHIRP_LEN	"Chirp Length (ms):"
#define CHIRP_COEFF1 "Chirp Constant Coefficient (Hz):"
#define CHIRP_COEFF2 "Chirp Linear Coefficient (Hz/micro-s):"
#define CHIRP_COEFF3 "Chirp Quadratic Coefficient (Hz/micro-s^2):"
#define GAIN_I		"Receiver Gain (dB) :"
#define LINE_FLAG	"Nought Line Flag:"
#define ANT_SQUINT1	"Antenna Squint Angle (millionths of deg):"
#define ANT_ELEV1	"Antenna Elevation Angle (millionths of deg):"
#define ANT_SQUINT2	"Antenna Squint Angle (millionths of deg):"
#define ANT_ELEV2	"Antenna Elevation Angle (millionths of deg):"
#define SLANT_DATA	"Slant Range to 1st Data Sample (m):"
#define DATA_WIND	"Data Record Window (ns):"
#define PLATF_POS	"Platform Position Parameters Update Flag:"
#define PLATF_LAT	"Platform Latitude (millionths of deg):"
#define PLATF_LON	"Platform Longitude (millionths of deg):"
#define PLATF_ALT	"Platform Altitude (m):"
#define PLATF_SPEED	"Platform Ground Speed (cm/s):"
#define PLATF_VELX	"Platform Velocity X (cm/s):"
#define PLATF_VELY	"Platform Velocity Y (cm/s):"
#define PLATF_VELZ	"Platform Velocity Z (cm/s):"
#define PLATF_ACCX	"Platform Acceleration X (cm/s^2):"
#define PLATF_ACCY	"Platform Acceleration Y (cm/s^2):"
#define PLATF_ACCZ	"Platform Acceleration Z (cm/s^2):"
#define PLATF_TRACK1 "Platform Track Angle (millionths of deg):"
#define PLATF_TRACK2 "Platform Track Angle (millionths of deg):"
#define PLATF_PITCH  "Platform Pitch Angle (millionths of deg):"
#define PLATF_ROLL	"Platform Roll Angle (millionths of deg):"
#define PLATF_YAW	"Platform Yaw Angle (millionths of deg):"
#define COUNTER		"Counter of PALSAR Frame:"
#define AUX_DATA	"PALSAR Auxiliary Data:"

/* Imagery Options File: Processed Data Record Headers */

#define SLANT_1ST	"Slant Range to 1st  Pixel:"
#define SLANT_MID	"Slant Range to Mid  Pixel:"
#define SLANT_LST	"Slant Range to Last Pixel:"
#define DOPPLER_1ST	"Doppler Centroid Value at 1st  Pixel:"
#define DOPPLER_MID	"Doppler Centroid Value at Mid  Pixel:"
#define DOPPLER_LST	"Doppler Centroid Value at Last Pixel:"
#define AZ_FM_1ST	"Azimuth FM Rate of 1st  Pixel:"
#define AZ_FM_MID	"Azimuth FM Rate of Mid  Pixel:"
#define AZ_FM_LST	"Azimuth FM Rate of Last Pixel:"
#define LOOK_ANGLE	"Look Angle of Nadir (millionths of deg):"
#define AZ_SQUINT	"Azimuth Squint ANgle (millionths of deg):"
#define GEO_REF		"Geographic Ref. Parameter Update Flag:"
#define LAT_1ST		"Latitude of 1st  Pixel:"
#define LAT_MID		"Latitude of Mid  Pixel:"
#define LAT_LST		"Latitude of Last Pixel:"
#define LON_1ST		"Longitude of 1st  Pixel:"
#define LON_MID		"Longitude of Mid  Pixel:"
#define LON_LST		"Longitude of Last Pixel:"
#define NORTHING_1ST "Northing of 1st  Pixel:"
#define NORTHING_LST "Northing of Last Pixel:"
#define EASTING_1ST	"Easting of 1st  Pixel:"
#define EASTING_LST	"Easting of Last Pixel:"
#define LINE_HEADING "Line Heading:"
	
/* Trailer File: File Descriptor Record Headers */

#define LOW_RECS	"Number of Low Res. Img Data Records:"
#define LOW_LEN		"Low Res. Img Data Record Length:"
#define LOW_PIX		"Number of Pixels of Low Res. Img Data:"
#define LOW_LINES	"Number of Lines of Low Res. Img Data:"
#define LOW_BYTES	"Number of Bytes per Sample:"

	
#endif /* ALOS_CEOS_RD_H */

